//#include <MMSYSTEM.H>
//#include <winbase.h>

#include <stdio.h>

#include "stdafx.h"
#include "BVA.h"

FILE *bvalog;

///////////////////////////////////////////////////////////////////////////////////
//
// BuenaVista Audio(BVA) buffer class
//
///////////////////////////////////////////////////////////////////////////////////
BVAbuffer::BVAbuffer(int len)
{
	size = len;
	pos = 0;
	relpos = 0;
	first = 1;
	buffer = new char[size];
	event = CreateEvent(NULL, TRUE, TRUE, "bufferEvent");
}

BVAbuffer::~BVAbuffer()
{
	delete buffer;
	CloseHandle(event);
}

int BVAbuffer::lock(int len, int po, char **ptr1, int *size1, char **ptr2, int *size2, int flag)
{
	int tmp, tmp1, tmp2;
	
	//tmp1 = GetTickCount();
	WaitForSingleObject(event, INFINITE);
	//tmp2 = GetTickCount();

#ifdef _DEBUG
//	fprintf(bvalog, "BVAbuffer lock: pos = %d, relpos = %d, len = %d, delay = %d\n", pos, relpos, len, tmp2-tmp1);
#endif
	
	ResetEvent(event);

	if(flag && CURRENTPOS)
		tmp = (pos + relpos) % size;
	else
		tmp = po;

	if(tmp + len >= size)
	{
		*size1 = size - tmp;
		*size2 = len - *size1;
	}
	else
	{
		*size1 = len;
		*size2 = 0;
	}

	*ptr1 = &buffer[tmp];
	*ptr2 = buffer;
	
	//relpos += len;

	return 1;
}

int BVAbuffer::get(int len, char **ptr1, int *size1, char **ptr2, int *size2)
{
	int tm;
	int relpostmp = relpos;

#ifdef _DEBUG
	tm = GetTickCount();
	fprintf(bvalog, "Get buffer, pos: %d, relpos: %d, time: %d ms\n", pos, relpos, tm);
#endif

	if(relpos == 0)
		first = 1;

	relpos = 0;
	lock(len, 0, ptr1, size1, ptr2, size2, CURRENTPOS);
	relpos = relpostmp;

	pos = (pos + len) % size;
	relpos -= len;
	if(relpos < 0)
		relpos = 0;
	
	return 1;
}

int BVAbuffer::unlock()
{
	SetEvent(event);
	return 1;
}

int BVAbuffer::play(char *buf, int len)
{
	char *buf1, *buf2;
	int len1, len2;
	int tm;

#ifdef _DEBUG
	tm = GetTickCount();
	fprintf(bvalog, "PLAY buffer, pos: %d, relpos: %d, time: %d ms\n", pos, relpos, tm);
#endif

	lock(2*len, 0, &buf1, &len1, &buf2, &len2, CURRENTPOS);
	if(len1 <= len)
	{
		memcpy(buf1, buf, len1);
		memcpy(buf2, &buf[len1], len - len1);
		ZeroMemory(&buf2[len - len1], len2 - (len - len1));
	}
	else
	{
		memcpy(buf1, buf, len);
		ZeroMemory(&buf1[len], len1-len);
		ZeroMemory(buf2, len2);
	}
	relpos += len;
	unlock();

	return 1;
}

void BVAbuffer::reset()
{
	WaitForSingleObject(event, INFINITE);
	ResetEvent(event);
	pos = 0;
	relpos = 0;
	first = 1;
	ZeroMemory(buffer, size);
	SetEvent(event);
}

int BVAbuffer::ready()
{


	if(!first)
		return 1;
	
	if(relpos >= DELAY)
	{
		first = 0;
		return 1;
	}

	if(relpos == 0)
	{
		first = 1;
	}

	return 0;
	
	//return 1;
}

///////////////////////////////////////////////////////////////////////////////////
//
// BuenaVista Audio(BVA) class
//
///////////////////////////////////////////////////////////////////////////////////
BVA::BVA()
{
	char out_event_name[20] = "write";
	int i;
	TIMECAPS tc;

#ifdef _DEBUG
	bvalog = fopen("bva.log", "w");
#endif

	size = DEFAULT_SIZE;
	primary = new short[size];
	ZeroMemory(primary, 2*size);

	for(i=0; i<MAX_STREAMS; i++)
		bvabuffer[i] = NULL;

	format = new WAVEFORMATEX;
	format->wFormatTag = WAVE_FORMAT_PCM;
	format->nChannels = 1;
	format->nSamplesPerSec = 8000;
    format->nAvgBytesPerSec = 16000; 
    format->nBlockAlign = 2; 
	format->wBitsPerSample = 16;
	format->cbSize = NULL;

	for(i=0; i < BUFFERS_NO; i++)
	{
		header[i].lpData = new char[2*DEFAULT_SIZE];
		header[i].dwBufferLength = 2*DEFAULT_SIZE;
		header[i].dwFlags = WHDR_DONE;
	}

	all = 0;
	late = 0;
	early = 0;
	toolate = 0;
	verylate = 0;
	
	// create write event
	hEvent = CreateEvent(NULL, TRUE, TRUE, out_event_name);
	active = 0;
	headerNo = 0;
	on = 0;
	streams = 0;

	if (timeGetDevCaps(&tc, sizeof(TIMECAPS)) != TIMERR_NOERROR) {
    // Error; application can't continue.
	}

	tres = min(max(tc.wPeriodMin, 1), tc.wPeriodMax);
	timeBeginPeriod(tres);  
}

BVA::~BVA()
{
	int i;

	timeEndPeriod(tres);
	
	for(i=0; i < MAX_STREAMS; i++)
		if(bvabuffer[i] != NULL)
			delete bvabuffer[i];
	delete primary;
	delete format;

#ifdef _DEBUG
//	fprintf(bvalog, "Chunks late: %d%%\n", late*100/all);
//	fprintf(bvalog, "Too late: %d%%\n", toolate*100/all);
//	fprintf(bvalog, "Very late: %d, %d%%\n", verylate, verylate*100/all);
//	fprintf(bvalog, "Early: %d%%\n", early*100/all);
	fclose(bvalog);
#endif

}

int BVA::init()
{
	int err;
	//char text[100];
	int i;

	first = 1;
	handle = NULL;
	headerNo = 0;
	active = 1;
	SetEvent(hEvent);


	for(i=0; i < MAX_STREAMS; i++)
		if(bvabuffer[i] != NULL)
			bvabuffer[i]->reset();

	if(handle == NULL)
	{
		err = waveOutOpen(&handle, WAVE_MAPPER, format, (DWORD)hEvent, NULL, 
			CALLBACK_EVENT);
		if(err != MMSYSERR_NOERROR)
		{
			//wsprintf(text, "Can't init audio out port %d", err);
			//MessageBox(NULL, text, "Error", MB_OK);
			return 0;
		}	
	}

	Sleep(800);

	on = 1;
#ifdef _DEBUG	
	fprintf(bvalog, "BVA init\n");
#endif
	
	TimerID = timeSetEvent(DEFAULT_SIZE/8, 1, Timer, (DWORD)this, TIME_PERIODIC);
    if(! TimerID)
        return 0;

	return 1;
}

void BVA::close()
{
	int err;
	int i;

	on = 0;
	Sleep(800);
	timeKillEvent(TimerID);
	err = waveOutReset(handle);
	if(err != MMSYSERR_NOERROR)
	{
	}
	
	
	for(i=0; i<BUFFERS_NO; i++)
		waveOutUnprepareHeader(handle, &header[i], sizeof(WAVEHDR));

	err = waveOutClose(handle);
	if(err != MMSYSERR_NOERROR)
	{
	}
	handle = NULL;

#ifdef _DEBUG	
	fprintf(bvalog, "BVA close\n");
#endif
}


int BVA::play(char *buffer)
{
	static int gap = 0;
	int nowait = 0;
	int err;
	int tmp1, tmp2;
	
	if(!first)
	{
		if(waveOutWrite(handle, &(header[headerNo]), sizeof(WAVEHDR))
			!= MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't write audio", "Error", MB_OK);
		
		headerNo = (headerNo + 1) % BUFFERS_NO;
		
		while(!(header[headerNo].dwFlags & WHDR_DONE))
			WaitForSingleObject(hEvent, INFINITE);

		if(waveOutUnprepareHeader(handle, &(header[headerNo]), sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
		{
			//MessageBox(NULL, "Can't unprepare audio out header", "Error", MB_OK);
		}
	}
	else
		first = 0;

	header[headerNo].dwFlags = 0L;
	header[headerNo].dwLoops = 0L;

	memcpy(header[headerNo].lpData, buffer, size*2); 

	err = waveOutPrepareHeader(handle, &(header[headerNo]), sizeof(WAVEHDR));
	if(err != MMSYSERR_NOERROR)
	{
		//MessageBox(NULL, "Can't prepare audio out header", "Error", MB_OK);
	}

	return 1;
}


int BVA::start()
{
	on = 1;
	return 1;
}

int BVA::stop()
{
	on = 0;
	return 1;
}

BVAbuffer *BVA::createBuffer(int len)
{
	int i = 0;
	BOOL found = FALSE;

	while((i < MAX_STREAMS) && (!found))
	{
		if(bvabuffer[i] == NULL)
			found = TRUE;
		else
			i++;
	}
	
	if(found)
	{
		bvabuffer[i] = new BVAbuffer(len);
		streams++;
	}
	else
		return NULL;

	return bvabuffer[i];
}

int BVA::deleteBuffer(BVAbuffer *buf)
{
	int i = 0;
	BOOL found = FALSE;

	while((i < MAX_STREAMS) && (!found))
	{
		if(bvabuffer[i] == buf)
			found = TRUE;
		else
			i++;
	}

	delete bvabuffer[i];
	bvabuffer[i] = 0;	
	streams--;

	return 1;
}




void BVA::reset()
{
	int i;

#ifdef _DEBUG	
			fprintf(bvalog, "RESET!!!!!!!!!!\n");
#endif

	waveOutReset(handle);
	WaitForSingleObject(hEvent, INFINITE);
	for(i=0; i<BUFFERS_NO; i++)
		waveOutUnprepareHeader(handle, &header[i], sizeof(WAVEHDR));
	first = 1;
	headerNo = 0;

	for(i=0; i < MAX_STREAMS; i++)
		if(bvabuffer[i] != NULL)
			bvabuffer[i]->reset();
}

/////////////////////////////////////////////////////////////////////////////////
//
// Main playing callback
//
void CALLBACK Timer(UINT wTimerID, UINT msg, 
    DWORD dwUser, DWORD dw1, DWORD dw2)  
{
	int i, j;
	char *buf1, *buf2;
	int len1, len2;
	int time = 0;
	int time1 = 0;
	int sched_time = 0;
	BVA *bva = (BVA *)dwUser;

		if(bva->on)
		{
			bva->play((char *)bva->primary);

			ZeroMemory(bva->primary, 2*bva->size);
			for(i=0; i < MAX_STREAMS; i++)
			{				
				if(bva->bvabuffer[i] != NULL)
				{
					if(bva->bvabuffer[i]->ready())
					{
						bva->bvabuffer[i]->get(2*bva->size, &buf1, &len1, &buf2, &len2);
						for(j = 0; j < len1/2; j++)
							bva->primary[j] += ((short *)buf1)[j]/bva->streams;
						if(len2 > 0)
						{
							for(j = 0; j < len2/2; j++)
								bva->primary[len1/2 + j] += ((short *)buf2)[j]/bva->streams;
							ZeroMemory(buf2, len2);
						}
						ZeroMemory(buf1, len1);
						bva->bvabuffer[i]->unlock();
					}
				}
				
			}
		
		
		}
}

