#include <vfw.h>

#include "stdafx.h"
//#include "h261.h"
//#include "h263.h"
//#include "video.h"
//#include "capture.h"
#include "VideoProtocol.h"
#include "convert.h"
#include "display.h"

//#define RGB24 1
#define BI_YUV9 961893977
#define FPS 10
#define START 14
#define STOP 15

class Capture
{
	private:
		CDialog *display;		// display dialog
		HWND capWnd;				// capture window handle
		BITMAPINFO *info;			// video format info
		HANDLE hCapEvent;			// event for synchronize capturing
		int size;
		Video *video;
		int state;
		char *buffer;

		int checkFormat(void);
		static LRESULT PASCAL frameClb(HWND hWnd, LPVIDEOHDR lpVHdr);

	public:
		int init(Video *v, HWND parent, int drv);
		void close();
		int start(void);
		void stop();
		int getFrame(unsigned char **frame, int *type, int *len);

};

Capture *cap;

int Capture::init(Video *v, HWND parent, int drv)
{
	CAPTUREPARMS params;
	
	cap = this;
	video = v;
	state == STOP;
	display = displayOpen(MAX_USERS);
	//displayHide(MAX_USERS);

	// capture window
	capWnd = capCreateCaptureWindow(
		(LPSTR) "Mirror",
		WS_CHILD /*| WS_VISIBLE*/,
		0, 0, 176, 144,
		display->m_hWnd,
		23);

	if(capWnd == NULL)
		return 0;

	// connect to capture driver
	if(!capDriverConnect(capWnd, drv))
	{
		DestroyWindow(capWnd);
		displayClose(MAX_USERS);
		return 0;
	}

	// set video format
	if(!checkFormat())
	{
		DestroyWindow(capWnd);
		displayClose(MAX_USERS);
		return 0;
	}

	// set callback for frame
	if(!capSetCallbackOnVideoStream(capWnd, frameClb))
		return 0;

	// set setup
	capCaptureGetSetup(capWnd, &params, sizeof(CAPTUREPARMS));
    params.dwRequestMicroSecPerFrame = 10 * FPS * 1000; 
    params.fMakeUserHitOKToCapture = FALSE; 
    params.wPercentDropForError = 10; 
    params.fYield = TRUE;
    params.wNumVideoRequested = 20; 
    params.fCaptureAudio = FALSE; 
    params.wNumAudioRequested = 1; 
    params.fLimitEnabled = FALSE; 
    params.wTimeLimit = 30; 
	params.fAbortLeftMouse = FALSE; 
    params.fAbortRightMouse = FALSE; 

	if(!capCaptureSetSetup(capWnd, &params, sizeof(CAPTUREPARMS)))
		return 0;

	capPreview(capWnd, FALSE);

	hCapEvent = CreateEvent(NULL, TRUE, TRUE, "capture event");

	size = info->bmiHeader.biWidth * info->bmiHeader.biHeight * info->bmiHeader.biBitCount / 8;
	buffer = new char[size];
	
	return 1;
}

void Capture::close()
{
	capSetCallbackOnFrame(capWnd, NULL);
	capDriverDisconnect(capWnd);
	DestroyWindow(capWnd);
	displayClose(MAX_USERS);
	GlobalFree(info);
	delete buffer;
}

int Capture::checkFormat(void)
{
	DWORD info_size;
	char text[100];

	// getting current format settings
	info_size = capGetVideoFormatSize(capWnd);
	info = (BITMAPINFO *)GlobalAlloc(GPTR, info_size);
	capGetVideoFormat(capWnd, info, info_size);

	switch(info->bmiHeader.biCompression)
	{
	case BI_RGB:
		if(info->bmiHeader.biBitCount != 24)
		{
			MessageBox(NULL, "Unsupported video format(wrong bits/pixel value). \
					Please, change your video format to RGB24 or YUV9.", "Error", MB_OK);
			return 0;
		}
		break;

	case BI_YUV9:
		break;

	default:
		MessageBox(NULL, "Unsupported video format. \
				Please, change your video format to RGB24 or YUV9.", "Error", MB_OK);
		return 0;
		break;
	}

	if((info->bmiHeader.biWidth < 176) && (info->bmiHeader.biHeight < 144))
	{
		MessageBox(NULL, "Unsupported video format(wrong picture size). \
			Please, change picture size to at least 176 x 144", "Error", MB_OK);
		return 0;
	}


	return 1;
}

int Capture::start(void)
{
	if(state == STOP)
	{
		state == STOP;
		if(!capCaptureSequenceNoFile(capWnd))
		{
			MessageBox(NULL, "Capture start error", "Error", MB_OK);
			return 0;
		}
		//displayShow(MAX_USERS);
	}

	return 1;
}

void Capture::stop()
{
	if(state == START)
	{
		capCaptureStop(capWnd);
		displayHide(MAX_USERS);
		state = STOP;
	}
}

LRESULT PASCAL Capture::frameClb(HWND hWnd, LPVIDEOHDR lpVHdr)
{
	WaitForSingleObject(cap->hCapEvent, INFINITE);
	ResetEvent(cap->hCapEvent);
	
	CopyMemory(cap->buffer, lpVHdr->lpData, cap->size);

	SetEvent(cap->hCapEvent);
	return TRUE;
}
 
int Capture::getFrame(unsigned char **frame, int *type, int *len)
{
	WaitForSingleObject(hCapEvent, INFINITE);
	ResetEvent(hCapEvent);
	
	switch(info->bmiHeader.biCompression)
	{
	case BI_RGB:
		*len = 3*176*144/2;
		*frame = (unsigned char *)malloc(*len);
		RGB24toYUV((unsigned char *)buffer, *frame, &(*frame[25344]), &(*frame[25344+6336]), info->bmiHeader.biWidth, info->bmiHeader.biHeight);
		break;

	case BI_YUV9:
		*len = size;
		*frame = new unsigned char[*len];
		CopyMemory(*frame, buffer, size);
		break;

	default:
		return 0;
		break;
	}

	SetEvent(hCapEvent);

	return 1;
}