/*********************************************************************
 *
 * File:         server.c
 *
 * Description:  NPAC Conferencing System API functions for server connection
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         12-17-96
 *
 *********************************************************************/
#include <stdio.h>
#include <winsock.h>

#include "common.h"

#define SERVER_ADDR "mirphak.npac.syr.edu"
#define SERVER_CFG_PATH "\\SERVERS.CFG"

extern FILE *log;
struct sockaddr_in serv_addr;
int serv_socket;
char server_name[50];
unsigned long server_id;


int ncsServerInit(char *sname)
{
	strcpy(server_name, sname);
	server_id = ncsGetHostID(server_name);
	PRINTF(log, "Manager: server %s, id %d\n", server_name, server_id);
	
	return server_id;
}

char *readServerMsg()
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF(log, "Manager: read server message\n");

  err = ncsReadMsgHeader(serv_socket, &header);
  if(err == 0)
    {
      PRINTF(log, "Mananger: server connection broken\n");
      return NULL;
    }
  if(err < 0)
    {
      PRINTF(log, "Manager: data reading error\n");
      return NULL;
    }

  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = ncsReadAll(serv_socket, msg, header.dataLength);
      if(err == 0)
		{
			PRINTF(log, "Manager: server connection broken\n");
			return NULL;
		}
      if(err < 0)
		{
			PRINTF(log, "Manager: data reading error\n");
			return NULL;
		}
    }
  else
    msg = NULL;
  
  //ncsEngineMessageHandle(header.tag, msg, header.dataLength);
  if(header.tag != indServerUsers)
  {
	  free(msg);
	  msg = NULL;
  }

  return msg;
}


int sendServerMessage(int hostid, short tag, char *data, int len)
{
  MsgHeader header;
  DWORD user_len = 20;

  if((serv_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
     {
       fprintf(stderr, "Manager: socket opening error\n");
       return -1;
     }

  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = hostid; 
  serv_addr.sin_port = htons(SERVER_PORT);

  if(connect(serv_socket, (struct sockaddr *)&serv_addr, sizeof(struct sockaddr_in)) < 0)
    {
      PRINTF(log, "Manager: cannot connect to server\n");
      closesocket(serv_socket);
      return -1;
    }

  header.tag = (unsigned char)tag;
  if(!GetUserName(header.sender.user, &user_len))
	  strcpy(header.sender.user, "unknown");
  header.sender.host = gethostid();
  header.dataLength = htonl(len);

  ncsSendControlMessage(serv_socket, &header, data, len);

  return 1;
  
}


int serverSendMsg(short tag, char *data, int len)
{
  return sendServerMessage(server_id, tag, data, len); 
}

void ncsServerAdd(void)
{
  serverSendMsg(reqServerAddUser, NULL, 0);
  closesocket(serv_socket);
}

void ncsServerDelete(void)
{
  serverSendMsg(reqServerDelUser, NULL, 0);
  closesocket(serv_socket);
}


char *ncsServerUsers()
{
	char *msg;

	msg = NULL;
	if(server_id)
	{
		if(serverSendMsg(reqServerUsers, NULL, 0) > 0)
			msg = readServerMsg();
		closesocket(serv_socket);
	}

	return msg;
}






















































































































