#include "stdafx.h"

#include <mmsystem.h>

#include "BVtango.h"
#include "BVtangoDlg.h"
#include "Conference.h"
#include "AddUsers.h"
#include "ChatDlg.h"
#include "global.h"
#include "daemon.h"
#include "options.h"
#include "PersonalInfo.h"
#include "tango.h"

extern Session session;

Conference::Conference(CBVtangoDlg *dlg)
{
	HANDLE hProcess;
	CPersonalInfo *info_dlg;
	
	hProcess = GetCurrentProcess();
	SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);

	main_dlg = dlg;
	restoreSettings();

	conferenceExists = 0;
	activePartsNum = 0;

	sscanf(AfxGetApp()->m_lpCmdLine, "%d %d", &tangoAID, &tangoPort);
	tangoSock = registerApp(tangoAID, tangoPort);

	//////////////////////////////////
	// Daemon thread creation
	hDaemonThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) daemonThread,	// thread function 
        this,									// argument to thread function 
        0,										// use default creation flags 
        &dwDaemonThreadId);						// returns the thread identifier 
	if (hDaemonThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	
	SetThreadPriority(hDaemonThread, THREAD_PRIORITY_LOWEST);


	ncsSessionInit();

	server = 0;
	//server = ncsServerInit(options.server_name);
	if(server)
		ncsServerAdd();

	main_dlg->SetStatus("Conferencing panel ready to use");

	tangoAck();
}

Conference::~Conference()
{
	if(server && !conferenceExists)
		ncsServerDelete();
	TerminateThread(hDaemonThread, 0);
	CloseHandle(hDaemonThread);
	saveSettings();
}

DWORD Conference::daemonThread(LPDWORD data)
{
	struct timeval timeout;
	fd_set set;
	int sockfd;

	timeout.tv_sec = 0;
	timeout.tv_usec = 200000;
	while(1)
	{
		FD_SET(((Conference *)data)->tangoSock, &set);
		if(select(FD_SETSIZE, &set, 0, 0, &timeout) > 0)
		{
			/* handle Tango message */
			if(FD_ISSET(((Conference *)data)->tangoSock, &set))
				((Conference *)data)->handleTangoMsg();
		}
	}

	return 1;
}

int Conference::saveSettings()
{
	char path[100];
	FILE *file;
	char *tmp;
	
	this->getStartPath(path);
	if(strcmp(path, "") == 0)
		strcat(path, "BuenaVista.cfg");
	else
		strcat(path, "\\BuenaVista.cfg");

	if(path[0] == '"')
		tmp = &path[1];
	else
		tmp = path;

	//MessageBox(NULL, tmp, "Info", MB_OK);
	file = fopen(tmp, "wb");
	if(file == NULL)
		return 0;	
	else
		if(fwrite(&options, sizeof(Options), 1, file) < 1)
			return 0;

	return 1;
}

int Conference::restoreSettings()
{
	char path[100];
	FILE *file;
	char *tmp;
	
	this->getStartPath(path);
	if(strcmp(path, "") == 0)
		strcat(path, "BuenaVista.cfg");
	else
		strcat(path, "\\BuenaVista.cfg");

	if(path[0] == '"')
		tmp = &path[1];
	else
		tmp = path;

	//MessageBox(NULL, tmp, "Info", MB_OK);
	file = fopen(tmp, "rb");
	if(file == NULL)
		standardSettings();
	else
		if(fread(&options, sizeof(Options), 1, file) < 1)
			standardSettings();

	return 1;
}

void Conference::standardSettings()
{
	strcpy(options.connection, "Modem 28.8");
	strcpy(options.server_name, "connecticut.npac.syr.edu");
	strcpy(options.user_name, "");
	strcpy(options.user_email, "");
	options.autoAccept = TRUE;

	options.audio.compression = GSM;
	options.audio.autoSwitch = FALSE;
	options.audio.manualSwitch = FALSE;
	options.audio.speakerTreshold = 20;
	options.audio.micTreshold = 20;
	options.audio.mode = HALF_DUPLEX;
	options.audio.volume = 50;
	options.audio.micGain = 50;
	options.audio.mute = FALSE;
	options.audio.micOff = FALSE;
	
	options.video.device = 0;
	options.video.compression = H263;
	options.video.intraMode = FALSE;
	options.video.limit = 5;
	options.video.maxlimit = 5;
	options.video.recOff = FALSE;
	
	options.audio.audioOut = NULL;

}


Session * Conference::getSessionInfo()
{
	return &session;
}


void Conference::getStartPath(char *path)
{
	int i;
	char *tmp;
	int found = 0;

	tmp = GetCommandLine();
	strcpy(path, tmp);
	i = strlen(path) - 1;
	while(!found && (i >= 0))
	{
		if(path[i] == '\\')
		{
			path[i] = 0;
			found = 1;
		}
		i--;
	}
	if(!found)
		strcpy(path, "");
}

void Conference::resetSession()
{
	ncsSessionReset();
	if(server)
		ncsServerAdd();
	conferenceExists = 0;
}


void Conference::tangoAck(void)
{
  struct Event *event;
  char tag = 100; 
  event = createEvent(CONTROL, 1, &tag); 
  sendEvent(tangoSock, event); 
  destroyEvent(event); 
}

/////////////////////////////////////////////////////////////////////////
//
// Tango message handler
//
/////////////////////////////////////////////////////////////////////////
void Conference::handleTangoMsg(void)
{
  TangoMsg *msg;
  struct Event *event;
  int no;
  
  event = receiveEvent(tangoSock);
  if(event == NULL)
  {
	  //MessageBox(NULL, "Manager: error reading Tango message, exiting...", "Error", MB_OK);
	  SendMessage(main_dlg->m_hWnd, WM_CLOSE, 0, NULL); 
	  return;
  }
  msg = (TangoMsg *)getData(event);
 switch(msg->tag)
    {
    case 1:
		SendMessage(main_dlg->m_hWnd, WM_COMMAND, 65536 * DS_JOIN + no, (long)&msg->part); 
		break;
      
    case 2:
		SendMessage(main_dlg->m_hWnd, WM_COMMAND, 65536 * DS_NEW, (long)&msg->part); 
		break;

    case 3:
		SendMessage(main_dlg->m_hWnd, WM_COMMAND, 65536 * DS_LEAVE, NULL); 
		break;

    case 4:
		SendMessage(main_dlg->m_hWnd, WM_COMMAND, 65536 * DS_DELETE, (long)&msg->part); 
		break;

    default:
		;//MessageBox(NULL, "CM: wrong tango message tag", "Error", MB_OK);
    }
  
 destroyEvent(event);
}
